@@ -1,10 +1,21 @@
-from optimade.models import DataType, StructureResource, ReferenceResource
+from optimade.models import (
+    DataType,
+    ErrorResponse,
+    StructureResource,
+    ReferenceResource,
+)
+from optimade.server.exceptions import POSSIBLE_ERRORS
 
 ENTRY_INFO_SCHEMAS = {
     "structures": StructureResource.schema,
     "references": ReferenceResource.schema,
 }
 
+ERROR_RESPONSES = {
+    err.status_code: {"model": ErrorResponse, "description": err.title}
+    for err in POSSIBLE_ERRORS
+}
+
 
 def retrieve_queryable_properties(
     schema: dict, queryable_properties: list = None
